// This code will take analog Humidity, Pressure, and Temperature sensors and output the readings
// to an SD/RTC shield.
// By: Christopher Gosch (modified version of code by Spencer McDonald)
// 5/27/14
// To speed up code, comment out LED Flashes

// Pin configuration
// Analog Pressure Sensor: A0
// Analog Humidty Sensor:  A1
// Analog Temperature Sensor: A2

// Adafruit SD/RTC Shield uses pins A4/A5, D10-13

// Libraries to include

#include "SD.h"                    // SD library

#include <Wire.h>                  // RTC library
#include "RTClib.h"

// Define Variables:

#define LEDpin 13  //Define a flashing LED that indicates sensor reading

const int chipSelect = 10; // Adafruit uses 10, Sparkfun uses 8
                                                  
File datalog;
char filename[] = "LOGGER00.csv";
RTC_DS1307 RTC;

int pressureSensor = 0;
float pressureSensorV = 0;
float psi = 0;

int analogTempSensor = 0;
float analogTempSensorVoltage = 0;
float analogTempSensorC = 0;

int humiditySensor = 0;
float humiditySensorV = 0;
float humiditySensorRH = 0;

void setup()
{
  Serial.begin(115200); // Opens the communication between Arduino and computer serial port
  
  Wire.begin();         // Begins RTC communication
  RTC.begin();
   
// Check if RTC is running, if not fix it

  if (! RTC.isrunning()) {
    Serial.println("RTC is NOT running!");
    // following line sets the RTC to the date & time this sketch was compiled
    RTC.adjust(DateTime(__DATE__, __TIME__));
  }


// Initalize SD Card  
  
  Serial.print("Intializing SD Card...");
  pinMode(10, OUTPUT); // Necessary for microSD shield
  
  if(!SD.begin(chipSelect)) 
  {
    Serial.println("Card failed, or not present");
    return;
  }
  Serial.println("Card initialized.");  
  
  Serial.print("Creating File...");
  

// Make a new file each time the arduino is powered

  for (uint8_t i = 0; i < 100; i++) 
  {
    filename[6] = i/10 + '0';
    filename[7] = i%10 + '0';
    if (! SD.exists(filename)) 
    {
      // only open a new file if it doesn't exist
      datalog = SD.open(filename, FILE_WRITE); 
      break;
    }
  }
  
  Serial.print("Logging to: ");
  Serial.println(filename);
  
  if(!datalog)
  {
  Serial.println("Couldn't Create File"); 
  return;
  }
  
// Print Header
  
  String Header = "RTC Date, RTC Time, Pressure (psi), Humidity (%RH), Temperature (C)";   
  
  datalog = SD.open(filename, FILE_WRITE);
  datalog.println(Header);
  Serial.println(Header); 
   
  datalog.close();
  
  digitalWrite(LEDpin, OUTPUT);
  digitalWrite(LEDpin, HIGH);
  delay(5000);
  digitalWrite(LEDpin, LOW);
  
}

void loop()
{
  DateTime now=RTC.now();
  
  datalog=SD.open(filename, FILE_WRITE);
  
   datalog.print(now.month(), DEC);
  datalog.print("/");
  datalog.print(now.day(), DEC);
  datalog.print("/");
  datalog.print(now.year(), DEC);
  datalog.print(",");
  datalog.print(now.hour(), DEC);
  datalog.print(":");
  datalog.print(now.minute(), DEC);
  datalog.print(":");
  datalog.print(now.second(), DEC);
  datalog.print(",");
 
  Serial.print(now.month(), DEC);
  Serial.print("/");
  Serial.print(now.day(), DEC);
  Serial.print("/");
  Serial.print(now.year(), DEC);
  Serial.print(", ");
  Serial.print(now.hour(), DEC);
  Serial.print(":");
  Serial.print(now.minute(), DEC);
  Serial.print(":");
  Serial.print(now.second(), DEC);
  Serial.print(", ");
  
  digitalWrite(LEDpin, HIGH);
  delay(500);
  digitalWrite(LEDpin, LOW);
  
   pressureSensor = analogRead(A0);                // Read the analog pin
  pressureSensorV = pressureSensor*(5.0/1024);    // Convert the digital number to voltage
  psi = (pressureSensorV - (0.1*5.0))/(4.0/15.0); // Convert the voltage to proper units
  Serial.print(psi,2);                            // Output the value
  Serial.print(",");                               // New line
  
  datalog.print(psi,2);                            //Print to datalog
  datalog.print(",");
  
  digitalWrite(LEDpin, HIGH);
  delay(100);
  digitalWrite(LEDpin, LOW);
  
  humiditySensor = analogRead(A1);                                // Read the analog pin
  humiditySensorV = humiditySensor*(5.0/1024);                    // Convert the digital number to voltage
  humiditySensorRH = (humiditySensorV - (5.0*0.16))/(5.0*0.0062); // Convert the voltage to proper units
  Serial.print(humiditySensorRH,2);                               // Output the value
  Serial.print(",");                                               // New line
  
  datalog.print(humiditySensorRH,2);                               // Output the value to file
  datalog.print(",");                                               // New line
  
  digitalWrite(LEDpin, HIGH);
  delay(250);
  digitalWrite(LEDpin, LOW);
  
  analogTempSensor = analogRead(A2);                        // Read the analog pin
  analogTempSensorVoltage = analogTempSensor*(5.0/1024);    // Convert the digital number to a voltage
  analogTempSensorC = (analogTempSensorVoltage - 0.5)/0.01; // Convert the voltage to proper units
  Serial.print(analogTempSensorC,2);                        // Output the value
  Serial.print(",");                                         // New line
  
  Serial.print(analogTempSensorC,2);                        // Output the value to file
  Serial.print(",");                                         // New line
  
  digitalWrite(LEDpin,HIGH);                                      // Flash LED to show sensor running
  delay(750);                                                    
  digitalWrite(LEDpin,LOW);
  
  datalog.flush();
  datalog.close();
  
  digitalWrite(LEDpin, HIGH);
  delay(1500);
  digitalWrite(LEDpin, LOW);

}
  
